@@ -20,11 +20,14 @@ from trove.cluster import models
 from trove.cluster import views
 from trove.common import apischema
 from trove.common import cfg
+from trove.common import clients
 from trove.common import exception
+from trove.common import glance as common_glance
 from trove.common.i18n import _
 from trove.common import notification
 from trove.common.notification import StartNotification
 from trove.common import pagination
+from trove.common import policy
 from trove.common import utils
 from trove.common import wsgi
 from trove.datastore import models as datastore_models
@@ -39,6 +42,11 @@ class ClusterController(wsgi.Controller):
     """Controller for cluster functionality."""
     schemas = apischema.cluster.copy()
 
+    @classmethod
+    def authorize_cluster_action(cls, context, cluster_rule_name, cluster):
+        policy.authorize_on_target(context, 'cluster:%s' % cluster_rule_name,
+                                   {'tenant': cluster.tenant_id})
+
     @classmethod
     def get_action_schema(cls, body, action_schema):
         action_type = list(body.keys())[0]
@@ -54,29 +62,40 @@ class ClusterController(wsgi.Controller):
     def action(self, req, body, tenant_id, id):
         LOG.debug(("Committing Action Against Cluster for "
                    "Tenant '%(tenant_id)s'\n"
-                   "req : '%(req)s'\n\nid : '%(id)s'\n\n") %
+                   "req : '%(req)s'\n\nid : '%(id)s'\n\n"),
                   {"req": req, "id": id, "tenant_id": tenant_id})
         if not body:
             raise exception.BadRequest(_("Invalid request body."))
+
         if len(body) != 1:
             raise exception.BadRequest(_("Action request should have exactly"
                                          " one action specified in body"))
         context = req.environ[wsgi.CONTEXT_KEY]
         cluster = models.Cluster.load(context, id)
+        if ('reset-status' in body and
+                'force_delete' not in body['reset-status']):
+            self.authorize_cluster_action(context, 'reset-status', cluster)
+        elif ('reset-status' in body and
+                'force_delete' in body['reset-status']):
+            self.authorize_cluster_action(context, 'force_delete', cluster)
+        else:
+            self.authorize_cluster_action(context, 'action', cluster)
         cluster.action(context, req, *next(iter(body.items())))
 
         view = views.load_view(cluster, req=req, load_servers=False)
         wsgi_result = wsgi.Result(view.data(), 202)
+
         return wsgi_result
 
     def show(self, req, tenant_id, id):
         """Return a single cluster."""
         LOG.debug(("Showing a Cluster for Tenant '%(tenant_id)s'\n"
-                   "req : '%(req)s'\n\nid : '%(id)s'\n\n") %
+                   "req : '%(req)s'\n\nid : '%(id)s'\n\n"),
                   {"req": req, "id": id, "tenant_id": tenant_id})
 
         context = req.environ[wsgi.CONTEXT_KEY]
         cluster = models.Cluster.load(context, id)
+        self.authorize_cluster_action(context, 'show', cluster)
         return wsgi.Result(views.load_view(cluster, req=req).data(), 200)
 
     def show_instance(self, req, tenant_id, cluster_id, instance_id):
@@ -85,13 +104,14 @@ class ClusterController(wsgi.Controller):
                    "'%(tenant_id)s'\n"
                    "req : '%(req)s'\n\n"
                    "cluster_id : '%(cluster_id)s'\n\n"
-                   "instance_id : '%(instance_id)s;\n\n") %
+                   "instance_id : '%(instance_id)s;\n\n"),
                   {"req": req, "tenant_id": tenant_id,
                    "cluster_id": cluster_id,
                    "instance_id": instance_id})
 
         context = req.environ[wsgi.CONTEXT_KEY]
         cluster = models.Cluster.load(context, cluster_id)
+        self.authorize_cluster_action(context, 'show_instance', cluster)
         instance = models.Cluster.load_instance(context, cluster.id,
                                                 instance_id)
         return wsgi.Result(views.ClusterInstanceDetailView(
@@ -100,11 +120,12 @@ class ClusterController(wsgi.Controller):
     def delete(self, req, tenant_id, id):
         """Delete a cluster."""
         LOG.debug(("Deleting a Cluster for Tenant '%(tenant_id)s'\n"
-                   "req : '%(req)s'\n\nid : '%(id)s'\n\n") %
+                   "req : '%(req)s'\n\nid : '%(id)s'\n\n"),
                   {"req": req, "id": id, "tenant_id": tenant_id})
 
         context = req.environ[wsgi.CONTEXT_KEY]
         cluster = models.Cluster.load(context, id)
+        self.authorize_cluster_action(context, 'delete', cluster)
         context.notification = notification.DBaaSClusterDelete(context,
                                                                request=req)
         with StartNotification(context, cluster_id=id):
@@ -114,12 +135,24 @@ class ClusterController(wsgi.Controller):
     def index(self, req, tenant_id):
         """Return a list of clusters."""
         LOG.debug(("Showing a list of clusters for Tenant '%(tenant_id)s'\n"
-                   "req : '%(req)s'\n\n") % {"req": req,
-                                             "tenant_id": tenant_id})
+                   "req : '%(req)s'\n\n"), {"req": req,
+                                            "tenant_id": tenant_id})
 
         context = req.environ[wsgi.CONTEXT_KEY]
-        if not context.is_admin and context.tenant != tenant_id:
-            raise exception.TroveOperationAuthError(tenant_id=context.tenant)
+
+        # This theoretically allows the Admin tenant list clusters for
+        # only one particular tenant as opposed to listing all clusters for
+        # for all tenants.
+        # * As far as I can tell this is the only call which actually uses the
+        #   passed-in 'tenant_id' for anything.
+        if not context.is_admin and context.project_id != tenant_id:
+            raise exception.TroveOperationAuthError(
+                tenant_id=context.project_id
+            )
+
+        # The rule checks that the currently authenticated tenant can perform
+        # the 'cluster-list' action.
+        policy.authorize_on_tenant(context, 'cluster:index')
 
         # load all clusters and instances for the tenant
         clusters, marker = models.Cluster.load_all(context, tenant_id)
@@ -130,16 +163,27 @@ class ClusterController(wsgi.Controller):
 
     def create(self, req, body, tenant_id):
         LOG.debug(("Creating a Cluster for Tenant '%(tenant_id)s'\n"
-                   "req : '%(req)s'\n\nbody : '%(body)s'\n\n") %
+                   "req : '%(req)s'\n\nbody : '%(body)s'\n\n"),
                   {"tenant_id": tenant_id, "req": req, "body": body})
 
         context = req.environ[wsgi.CONTEXT_KEY]
+        policy.authorize_on_tenant(context, 'cluster:create')
+
         name = body['cluster']['name']
         datastore_args = body['cluster'].get('datastore', {})
         datastore, datastore_version = (
             datastore_models.get_datastore_version(**datastore_args))
 
-        # TODO(saurabhs): add extended_properties to apischema
+        # Since Victoria, guest agent uses docker.
+        # Get image_id from glance if image_id in datastore_versions table
+        # is NULL.
+        image_id = None
+        if not datastore_version.image_id:
+            glance_client = clients.create_glance_client(context)
+            image_id = common_glance.get_image_id(
+                glance_client, datastore_version.image_id,
+                datastore_version.image_tags)
+
         extended_properties = body['cluster'].get('extended_properties', {})
 
         try:
@@ -174,6 +218,7 @@ class ClusterController(wsgi.Controller):
                               "volume_type": volume_type,
                               "nics": nics,
                               "availability_zone": availability_zone,
+                              'region_name': node.get('region_name'),
                               "modules": modules})
 
         locality = body['cluster'].get('locality')
@@ -184,7 +229,9 @@ class ClusterController(wsgi.Controller):
                                    (locality,
                                     "', '".join(locality_domain)))
             if locality not in locality_domain:
-                raise exception.BadRequest(msg=locality_domain_msg)
+                raise exception.BadRequest(message=locality_domain_msg)
+
+        configuration = body['cluster'].get('configuration')
 
         context.notification = notification.DBaaSClusterCreate(context,
                                                                request=req)
@@ -193,7 +240,8 @@ class ClusterController(wsgi.Controller):
             cluster = models.Cluster.create(context, name, datastore,
                                             datastore_version, instances,
                                             extended_properties,
-                                            locality)
+                                            locality, configuration,
+                                            image_id)
         cluster.locality = locality
         view = views.load_view(cluster, req=req, load_servers=False)
         return wsgi.Result(view.data(), 200)
