@@ -28,6 +28,7 @@ from oslo_log import versionutils
 from oslo_middleware import cors
 from osprofiler import opts as profiler
 
+from trove.common import cache
 from trove.common.i18n import _
 from trove.version import version_info as version
 
@@ -484,7 +485,7 @@ common_opts = [
                     'Keystone v3 API with big service catalogs).'),
     cfg.StrOpt('conductor_manager', default='trove.conductor.manager.Manager',
                help='Qualified class name to use for conductor manager.'),
-    cfg.StrOpt('network_driver', default='trove.network.nova.NovaNetwork',
+    cfg.StrOpt('network_driver', default='trove.network.neutron.NeutronDriver',
                help="Describes the actual network manager used for "
                     "the management of network attributes "
                     "(security groups, floating IPs, etc.)."),
@@ -536,7 +537,16 @@ common_opts = [
     cfg.BoolOpt(
         'enable_volume_az', default=False,
         help='If true create the volume in the same availability-zone as the '
-             'instance')
+             'instance'),
+    cfg.StrOpt(
+        'replica_snapshot_driver',
+        choices=['cinder', 'swift'],
+        help='by default, this is the same as storage_strategy. it may be '
+             'useful to set to different driver to avoid the limitation of '
+             'backup drivers. for example, we can set to cinder when '
+             'storage_strate is swift to increase the backup speed, and '
+             'set to swift when storage_strate is cinder to support backup '
+             'cross az or region'),
 ]
 
 # Mysql
@@ -1611,6 +1621,7 @@ def list_opts():
         (network_group, network_opts),
         (service_credentials_group, service_credentials_opts),
         (guest_agent_group, guest_agent_opts),
+        (cache.PORTS_CACHE_GROUP, cache.PORTS_CACHE_OPTS)
     ]
 
     return keystone_middleware_opts + keystone_loading_opts + trove_opts
