@@ -17,27 +17,22 @@ import abc
 
 from lxml import etree
 from oslo_log import log as logging
-from oslo_utils import encodeutils
 import routes
-import six
 import stevedore
 import webob.dec
 import webob.exc
 
 from trove.common import base_exception as exception
 from trove.common import base_wsgi
-from trove.common import cfg
 from trove.common.i18n import _
 from trove.common import wsgi
 
 LOG = logging.getLogger(__name__)
-CONF = cfg.CONF
 DEFAULT_XMLNS = "http://docs.openstack.org/trove"
 XMLNS_ATOM = "http://www.w3.org/2005/Atom"
 
 
-@six.add_metaclass(abc.ABCMeta)
-class ExtensionDescriptor(object):
+class ExtensionDescriptor(object, metaclass=abc.ABCMeta):
     """Base class that defines the contract for extensions.
 
     Note that you don't have to derive from this class to have a valid
@@ -399,8 +394,7 @@ class ExtensionManager(object):
             LOG.debug('Ext namespace: %s', extension.get_namespace())
             LOG.debug('Ext updated: %s', extension.get_updated())
         except AttributeError as ex:
-            LOG.exception(_("Exception loading extension: %s"),
-                          encodeutils.exception_to_unicode(ex))
+            LOG.exception("Exception loading extension: %s", ex)
             return False
         return True
 
