@@ -19,8 +19,8 @@ import inspect
 import os
 import shlex
 import shutil
-import uuid
 import urllib.parse as urlparse
+import uuid
 
 from eventlet.timeout import Timeout
 import jinja2
@@ -30,9 +30,9 @@ from oslo_service import loopingcall
 from oslo_utils.encodeutils import safe_encode
 from oslo_utils import importutils
 from oslo_utils import strutils
-from passlib import pwd
 
 from trove.common import cfg
+from trove.common import crypto_utils
 from trove.common import exception
 from trove.common.i18n import _
 
@@ -175,7 +175,7 @@ class MethodInspector(object):
 
     @cached_property
     def argspec(self):
-        return inspect.getargspec(self._func)
+        return inspect.getfullargspec(self._func)
 
     def __str__(self):
         optionals = ["[{0}=<{0}>]".format(k) for k, v in self.optional_args]
@@ -298,7 +298,7 @@ def generate_random_password(password_length=None):
         password_length or
         cfg.get_configuration_property('default_password_length')
     )
-    return pwd.genword(length=password_length)
+    return crypto_utils.generate_random_key(length=password_length)
 
 
 def try_recover(func):
