@@ -13,14 +13,16 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from functools import reduce
 import inspect
 import operator
 import os
+from pathlib import Path
+import pwd
 import re
 import stat
 import tempfile
 
-from functools import reduce
 from oslo_concurrency.processutils import UnknownArgumentError
 
 from trove.common import exception
@@ -56,13 +58,18 @@ def read_file(path, codec=IdentityCodec(), as_root=False, decode=True):
     :raises:                :class:`UnprocessableEntity` if codec not given.
     """
     if path and exists(path, is_directory=False, as_root=as_root):
-        if as_root:
-            return _read_file_as_root(path, codec, decode=decode)
+        if decode:
+            open_flag = 'r'
+            convert_func = codec.deserialize
+        else:
+            open_flag = 'rb'
+            convert_func = codec.serialize
 
-        with open(path, 'r') as fp:
-            if decode:
-                return codec.deserialize(fp.read())
-            return codec.serialize(fp.read())
+        if as_root:
+            return _read_file_as_root(path, open_flag, convert_func)
+
+        with open(path, open_flag) as fp:
+            return convert_func(fp.read())
 
     raise exception.UnprocessableEntity(_("File does not exist: %s") % path)
 
@@ -97,28 +104,47 @@ def exists(path, is_directory=False, as_root=False):
     return found
 
 
-def _read_file_as_root(path, codec, decode=True):
+def find_executable(executable, path=None):
+    """Finds a location of an executable in the locations listed in 'path'
+
+    :param executable          File to search.
+    :type executable           string
+
+    :param path                Lookup directories separated by a path
+                               separartor.
+    :type path                 string
+    """
+    if path is None:
+        path = os.environ.get('PATH', os.defpath)
+    dirs = path.split(os.pathsep)
+    for directory in dirs:
+        exec_path = os.path.join(directory, executable)
+        if os.path.isfile(exec_path) and os.access(exec_path, os.X_OK):
+            return exec_path
+    return None
+
+
+def _read_file_as_root(path, open_flag, convert_func):
     """Read a file as root.
 
     :param path                Path to the written file.
     :type path                 string
 
-    :param codec:              A codec used to transform the data.
-    :type codec:               StreamCodec
+    :param open_flag:          The flag for opening a file
+    :type open_flag:           string
 
-    :param decode:             Should the codec decode the data.
-    :type decode:              boolean
+    :param convert_func:       The function for converting data.
+    :type convert_func:        callable
     """
-    with tempfile.NamedTemporaryFile() as fp:
+    with tempfile.NamedTemporaryFile(open_flag) as fp:
         copy(path, fp.name, force=True, dereference=True, as_root=True)
         chmod(fp.name, FileMode.ADD_READ_ALL(), as_root=True)
-        if decode:
-            return codec.deserialize(fp.read())
-        return codec.serialize(fp.read())
+        return convert_func(fp.read())
 
 
 def write_file(path, data, codec=IdentityCodec(), as_root=False, encode=True):
     """Write data into file using a given codec.
+
     Overwrite any existing contents.
     The written file can be read back into its original
     form by 'read_file'.
@@ -141,20 +167,24 @@ def write_file(path, data, codec=IdentityCodec(), as_root=False, encode=True):
     :raises:                   :class:`UnprocessableEntity` if path not given.
     """
     if path:
-        if as_root:
-            _write_file_as_root(path, data, codec, encode=encode)
+        if encode:
+            open_flag = 'w'
+            convert_func = codec.serialize
         else:
-            with open(path, 'w') as fp:
-                if encode:
-                    fp.write(codec.serialize(data))
-                else:
-                    fp.write(codec.deserialize(data))
+            open_flag = 'wb'
+            convert_func = codec.deserialize
+
+        if as_root:
+            _write_file_as_root(path, data, open_flag, convert_func)
+        else:
+            with open(path, open_flag) as fp:
+                fp.write(convert_func(data))
                 fp.flush()
     else:
         raise exception.UnprocessableEntity(_("Invalid path: %s") % path)
 
 
-def _write_file_as_root(path, data, codec, encode=True):
+def _write_file_as_root(path, data, open_flag, convert_func):
     """Write a file as root. Overwrite any existing contents.
 
     :param path                Path to the written file.
@@ -163,19 +193,16 @@ def _write_file_as_root(path, data, codec, encode=True):
     :param data:               An object representing the file contents.
     :type data:                StreamCodec
 
-    :param codec:              A codec used to transform the data.
-    :type codec:               StreamCodec
+    :param open_flag:          The flag for opening a file
+    :type open_flag:           string
 
-    :param encode:             Should the codec encode the data.
-    :type encode:              boolean
+    :param convert_func:       The function for converting data.
+    :type convert_func:        callable
     """
     # The files gets removed automatically once the managing object goes
     # out of scope.
-    with tempfile.NamedTemporaryFile('w', delete=False) as fp:
-        if encode:
-            fp.write(codec.serialize(data))
-        else:
-            fp.write(codec.deserialize(data))
+    with tempfile.NamedTemporaryFile(open_flag, delete=False) as fp:
+        fp.write(convert_func(data))
         fp.flush()
         fp.close()  # Release the resource before proceeding.
         copy(fp.name, path, force=True, as_root=True)
@@ -454,11 +481,46 @@ def service_discovery(service_candidates):
     return result
 
 
-def create_directory(dir_path, user=None, group=None, force=True, **kwargs):
+def execute_shell_cmd(cmd, options, *args, **kwargs):
+    """Execute a given shell command passing it
+    given options (flags) and arguments.
+
+    Takes optional keyword arguments:
+    :param as_root:        Execute as root.
+    :type as_root:         boolean
+
+    :param timeout:        Number of seconds if specified,
+                           default if not.
+                           There is no timeout if set to None.
+    :type timeout:         integer
+
+    :raises:               class:`UnknownArgumentError` if passed unknown args.
+    """
+
+    exec_args = {}
+    if kwargs.pop('as_root', False):
+        exec_args['run_as_root'] = True
+        exec_args['root_helper'] = 'sudo'
+
+    if 'timeout' in kwargs:
+        exec_args['timeout'] = kwargs.pop('timeout')
+
+    exec_args['shell'] = kwargs.pop('shell', False)
+
+    if kwargs:
+        raise UnknownArgumentError(_("Got unknown keyword args: %r") % kwargs)
+
+    cmd_flags = _build_command_options(options)
+    cmd_args = cmd_flags + list(args)
+    stdout, stderr = utils.execute_with_timeout(cmd, *cmd_args, **exec_args)
+    return stdout
+
+
+def ensure_directory(dir_path, user=None, group=None, force=True, **kwargs):
     """Create a given directory and update its ownership
     (recursively) to the given user and group if any.
 
-    seealso:: _execute_shell_cmd for valid optional keyword arguments.
+    seealso:: execute_shell_cmd for valid optional keyword arguments.
 
     :param dir_path:        Path to the created directory.
     :type dir_path:         string
@@ -488,7 +550,7 @@ def create_directory(dir_path, user=None, group=None, force=True, **kwargs):
 def chown(path, user, group, recursive=True, force=False, **kwargs):
     """Changes the owner and group of a given file.
 
-    seealso:: _execute_shell_cmd for valid optional keyword arguments.
+    seealso:: execute_shell_cmd for valid optional keyword arguments.
 
     :param path:         Path to the modified file.
     :type path:          string
@@ -518,7 +580,7 @@ def chown(path, user, group, recursive=True, force=False, **kwargs):
 
     owner_group_modifier = _build_user_group_pair(user, group)
     options = (('f', force), ('R', recursive))
-    _execute_shell_cmd('chown', options, owner_group_modifier, path, **kwargs)
+    execute_shell_cmd('chown', options, owner_group_modifier, path, **kwargs)
 
 
 def _build_user_group_pair(user, group):
@@ -534,17 +596,18 @@ def _create_directory(dir_path, force=True, **kwargs):
     :param force:           No error if existing, make parent directories
                             as needed.
     :type force:            boolean
+    :param as_root: Run as root user, default: False.
     """
 
     options = (('p', force),)
-    _execute_shell_cmd('mkdir', options, dir_path, **kwargs)
+    execute_shell_cmd('mkdir', options, dir_path, **kwargs)
 
 
 def chmod(path, mode, recursive=True, force=False, **kwargs):
     """Changes the mode of a given file.
 
     :seealso: Modes for more information on the representation of modes.
-    :seealso: _execute_shell_cmd for valid optional keyword arguments.
+    :seealso: execute_shell_cmd for valid optional keyword arguments.
 
     :param path:            Path to the modified file.
     :type path:             string
@@ -567,7 +630,7 @@ def chmod(path, mode, recursive=True, force=False, **kwargs):
     if path:
         options = (('f', force), ('R', recursive))
         shell_modes = _build_shell_chmod_mode(mode)
-        _execute_shell_cmd('chmod', options, shell_modes, path, **kwargs)
+        execute_shell_cmd('chmod', options, shell_modes, path, **kwargs)
     else:
         raise exception.UnprocessableEntity(
             _("Cannot change mode of a blank file."))
@@ -577,7 +640,7 @@ def change_user_group(user, group, append=True, add_group=True, **kwargs):
     """Adds a user to groups by using the usermod linux command with -a and
     -G options.
 
-    seealso:: _execute_shell_cmd for valid optional keyword arguments.
+    seealso:: execute_shell_cmd for valid optional keyword arguments.
 
     :param user:            Username.
     :type user:             string
@@ -606,7 +669,7 @@ def change_user_group(user, group, append=True, add_group=True, **kwargs):
         raise exception.UnprocessableEntity(_("Missing group."))
 
     options = (('a', append), ('G', add_group))
-    _execute_shell_cmd('usermod', options, group, user, **kwargs)
+    execute_shell_cmd('usermod', options, group, user, **kwargs)
 
 
 def _build_shell_chmod_mode(mode):
@@ -642,7 +705,7 @@ def _build_shell_chmod_mode(mode):
 def remove(path, force=False, recursive=True, **kwargs):
     """Remove a given file or directory.
 
-    :seealso: _execute_shell_cmd for valid optional keyword arguments.
+    :seealso: execute_shell_cmd for valid optional keyword arguments.
 
     :param path:            Path to the removed file.
     :type path:             string
@@ -658,7 +721,7 @@ def remove(path, force=False, recursive=True, **kwargs):
 
     if path:
         options = (('f', force), ('R', recursive))
-        _execute_shell_cmd('rm', options, path, **kwargs)
+        execute_shell_cmd('rm', options, path, **kwargs)
     else:
         raise exception.UnprocessableEntity(_("Cannot remove a blank file."))
 
@@ -668,7 +731,7 @@ def move(source, destination, force=False, **kwargs):
     Move attempts to preserve the original ownership, permissions and
     timestamps.
 
-    :seealso: _execute_shell_cmd for valid optional keyword arguments.
+    :seealso: execute_shell_cmd for valid optional keyword arguments.
 
     :param source:          Path to the source location.
     :type source:           string
@@ -689,16 +752,17 @@ def move(source, destination, force=False, **kwargs):
         raise exception.UnprocessableEntity(_("Missing destination path."))
 
     options = (('f', force),)
-    _execute_shell_cmd('mv', options, source, destination, **kwargs)
+    execute_shell_cmd('mv', options, source, destination, **kwargs)
 
 
 def copy(source, destination, force=False, preserve=False, recursive=True,
          dereference=False, **kwargs):
     """Copy a given file or directory to another location.
+
     Copy does NOT attempt to preserve ownership, permissions and timestamps
     unless the 'preserve' option is enabled.
 
-    :seealso: _execute_shell_cmd for valid optional keyword arguments.
+    :seealso: execute_shell_cmd for valid optional keyword arguments.
 
     :param source:          Path to the source location.
     :type source:           string
@@ -730,7 +794,7 @@ def copy(source, destination, force=False, preserve=False, recursive=True,
 
     options = (('f', force), ('p', preserve), ('R', recursive),
                ('L', dereference))
-    _execute_shell_cmd('cp', options, source, destination, **kwargs)
+    execute_shell_cmd('cp', options, source, destination, **kwargs)
 
 
 def get_bytes_free_on_fs(path):
@@ -767,7 +831,7 @@ def list_files_in_directory(root_dir, recursive=False, pattern=None,
         if pattern:
             cmd_args.extend(['-regextype', 'posix-extended',
                              '-regex', os.path.join('.*', pattern) + '$'])
-        files = _execute_shell_cmd('find', [], *cmd_args, as_root=True)
+        files = execute_shell_cmd('find', [], *cmd_args, as_root=True)
         return {fp for fp in files.splitlines()}
 
     return {os.path.abspath(os.path.join(root, name))
@@ -777,39 +841,6 @@ def list_files_in_directory(root_dir, recursive=False, pattern=None,
             if not pattern or re.match(pattern, name)}
 
 
-def _execute_shell_cmd(cmd, options, *args, **kwargs):
-    """Execute a given shell command passing it
-    given options (flags) and arguments.
-
-    Takes optional keyword arguments:
-    :param as_root:        Execute as root.
-    :type as_root:         boolean
-
-    :param timeout:        Number of seconds if specified,
-                           default if not.
-                           There is no timeout if set to None.
-    :type timeout:         integer
-
-    :raises:               class:`UnknownArgumentError` if passed unknown args.
-    """
-
-    exec_args = {}
-    if kwargs.pop('as_root', False):
-        exec_args['run_as_root'] = True
-        exec_args['root_helper'] = 'sudo'
-
-    if 'timeout' in kwargs:
-        exec_args['timeout'] = kwargs.pop('timeout')
-
-    if kwargs:
-        raise UnknownArgumentError(_("Got unknown keyword args: %r") % kwargs)
-
-    cmd_flags = _build_command_options(options)
-    cmd_args = cmd_flags + list(args)
-    stdout, stderr = utils.execute_with_timeout(cmd, *cmd_args, **exec_args)
-    return stdout
-
-
 def _build_command_options(options):
     """Build a list of flags from given pairs (option, is_enabled).
     Each option is prefixed with a single '-'.
@@ -817,3 +848,100 @@ def _build_command_options(options):
     """
 
     return ['-' + item[0] for item in options if item[1]]
+
+
+def get_device(path, as_root=False):
+    """Get the device that a given path exists on."""
+    stdout = execute_shell_cmd('df', [], path, as_root=as_root)
+    return stdout.splitlines()[1].split()[0]
+
+
+def is_mount(path):
+    """Check if the given directory path is a mountpoint. Try the standard
+    ismount first. This fails if the path is not accessible though, so resort
+    to checking as the root user (which is slower).
+    """
+    if os.access(path, os.R_OK):
+        return os.path.ismount(path)
+    if not exists(path, is_directory=True, as_root=True):
+        return False
+    directory_dev = get_device(path, as_root=True)
+    parent_dev = get_device(os.path.join(path, '..'), as_root=True)
+    return directory_dev != parent_dev
+
+
+def get_current_user():
+    """Returns name of the current OS user"""
+    return pwd.getpwuid(os.getuid())[0]
+
+
+def create_user(user_name, user_id, group_name=None, group_id=None):
+    group_name = group_name or user_name
+    group_id = group_id or user_id
+
+    try:
+        execute_shell_cmd('groupadd', [], '--gid', group_id, group_name,
+                          as_root=True)
+    except exception.ProcessExecutionError as err:
+        if 'already exists' not in err.stderr:
+            raise exception.UnprocessableEntity(
+                'Failed to add group %s, error: %s' % (group_name, err.stderr)
+            )
+
+    try:
+        execute_shell_cmd('useradd', [], '--uid', user_id, '--gid', group_id,
+                          '-M', user_name, as_root=True)
+    except exception.ProcessExecutionError as err:
+        if 'already exists' not in err.stderr:
+            raise exception.UnprocessableEntity(
+                'Failed to add user %s, error: %s' % (user_name, err.stderr)
+            )
+
+
+def remove_dir_contents(folder):
+    """Remove all the files and sub-directories but keep the folder.
+
+    Use shell=True here because shell=False doesn't support '*'
+    """
+    path = os.path.join(folder, '*')
+    execute_shell_cmd(f'rm -rf {path}', [], shell=True, as_root=True)
+
+
+def get_dir_size(path):
+    """Get the directory size in bytes."""
+    root_directory = Path(path)
+    return sum(f.stat().st_size for f in root_directory.glob('**/*')
+               if f.is_file())
+
+
+def get_filesystem_size(path):
+    """Get size(bytes) of a mounted filesystem the given path locates.
+
+    path is the pathname of any file within the mounted filesystem.
+    """
+    ret = os.statvfs(path)
+    return ret.f_blocks * ret.f_frsize
+
+
+def sync(path):
+    """Sync path files to disk."""
+    if not exists(path, is_directory=True, as_root=True):
+        raise exception.UnprocessableEntity(
+            _("Invalid path: %s") % path)
+    execute_shell_cmd('sync', [], path, as_root=True)
+
+
+def fsfreeze(mount_point):
+    """Freeze mount point file system"""
+    if not exists(mount_point, is_directory=True, as_root=True):
+        raise exception.UnprocessableEntity(
+            _("Invalid path: %s") % mount_point)
+    execute_shell_cmd('fsfreeze', [('f', True)], mount_point, as_root=True)
+
+
+def fsunfreeze(mount_point):
+    """Unfreeze mount point file system"""
+    if not exists(mount_point, is_directory=True, as_root=True):
+        raise exception.UnprocessableEntity(
+            _("Invalid path: %s") % mount_point)
+    execute_shell_cmd('fsfreeze', [('u', True)], mount_point, as_root=True)
