@@ -13,7 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
 import uuid
 
 from novaclient import exceptions as nova_exceptions
@@ -21,8 +21,8 @@ from novaclient import exceptions as nova_exceptions
 from trove.cluster import models
 from trove.cluster import tasks
 from trove.common import cfg
+from trove.common import clients
 from trove.common import exception
-from trove.common import remote
 from trove.common.strategies.cluster.experimental.mongodb import api
 from trove.instance import models as inst_models
 from trove.instance import tasks as inst_tasks
@@ -37,10 +37,14 @@ CONF = cfg.CONF
 class FakeOptGroup(object):
     def __init__(self, num_config_servers_per_cluster=3,
                  num_query_routers_per_cluster=1,
+                 config_servers_volume_size=10,
+                 query_routers_volume_size=10,
                  cluster_secure=True, volume_support=True,
                  device_path='/dev/vdb'):
         self.num_config_servers_per_cluster = num_config_servers_per_cluster
         self.num_query_routers_per_cluster = num_query_routers_per_cluster
+        self.config_servers_volume_size = config_servers_volume_size
+        self.query_routers_volume_size = query_routers_volume_size
         self.cluster_secure = cluster_secure
         self.volume_support = volume_support
         self.device_path = device_path
@@ -71,7 +75,7 @@ class MongoDBClusterTest(trove_testtools.TestCase):
         self.manager = mock.Mock()
         self.cluster.manager = self.manager
         self.volume_support = CONF.get('mongodb').volume_support
-        self.remote_nova = remote.create_nova_client
+        self.remote_nova = clients.create_nova_client
         self.instances = [
             {'volume_size': 1, 'flavor_id': '1234',
              'nics': [{"net-id": "foo-bar"}],
@@ -110,7 +114,7 @@ class MongoDBClusterTest(trove_testtools.TestCase):
                           self.datastore_version,
                           instance, {}, None, None)
 
-    @mock.patch.object(remote, 'create_nova_client')
+    @mock.patch.object(clients, 'create_nova_client')
     def test_create_invalid_flavor_specified(self, mock_client):
         (mock_client.return_value.flavors.get) = mock.Mock(
             side_effect=nova_exceptions.NotFound(
@@ -123,7 +127,7 @@ class MongoDBClusterTest(trove_testtools.TestCase):
                           self.datastore_version,
                           self.instances, {}, None, None)
 
-    @mock.patch.object(remote, 'create_nova_client')
+    @mock.patch.object(clients, 'create_nova_client')
     def test_create_flavor_not_equal(self, mock_client):
         instances = self.instances
         instances[0]['flavor_id'] = '4321'
@@ -137,7 +141,7 @@ class MongoDBClusterTest(trove_testtools.TestCase):
                           self.datastore_version,
                           instances, {}, None, None)
 
-    @mock.patch.object(remote, 'create_nova_client')
+    @mock.patch.object(clients, 'create_nova_client')
     def test_create_volume_not_equal(self, mock_client):
         instances = self.instances
         instances[0]['volume_size'] = 2
@@ -151,7 +155,7 @@ class MongoDBClusterTest(trove_testtools.TestCase):
                           self.datastore_version,
                           instances, {}, None, None)
 
-    @mock.patch.object(remote, 'create_nova_client')
+    @mock.patch.object(clients, 'create_nova_client')
     def test_create_volume_not_specified(self, mock_client):
         instances = [
             {'flavor_id': '1234',
@@ -173,7 +177,7 @@ class MongoDBClusterTest(trove_testtools.TestCase):
                           self.datastore_version,
                           instances, {}, None, None)
 
-    @mock.patch.object(remote, 'create_nova_client')
+    @mock.patch.object(clients, 'create_nova_client')
     @mock.patch.object(api, 'CONF')
     def test_create_storage_specified_with_no_volume_support(self,
                                                              mock_conf,
@@ -190,12 +194,31 @@ class MongoDBClusterTest(trove_testtools.TestCase):
                           self.datastore_version,
                           self.instances, {}, None, None)
 
+    @mock.patch.object(task_api, 'load')
+    @mock.patch.object(inst_models.Instance, 'create')
+    @mock.patch.object(models.DBCluster, 'create')
+    @mock.patch.object(clients, 'create_neutron_client')
+    @mock.patch.object(clients, 'create_nova_client')
+    @mock.patch.object(api, 'check_quotas')
+    def test_create_validate_volumes_deltas(self, mock_check_quotas, *args):
+        extended_properties = {
+            "configsvr_volume_size": 5,
+            "mongos_volume_size": 7}
+        self.cluster.create(mock.Mock(),
+                            self.cluster_name,
+                            self.datastore,
+                            self.datastore_version,
+                            self.instances,
+                            extended_properties, None, None)
+        deltas = {'instances': 7, 'volumes': 25}  # volumes=1*3+5*3+7*1
+        mock_check_quotas.assert_called_with(mock.ANY, deltas)
+
     @mock.patch.object(task_api, 'load')
     @mock.patch.object(inst_models.Instance, 'create')
     @mock.patch.object(models.DBCluster, 'create')
     @mock.patch.object(QUOTAS, 'check_quotas')
-    @mock.patch.object(remote, 'create_nova_client')
-    @mock.patch.object(remote, 'create_neutron_client')
+    @mock.patch.object(clients, 'create_nova_client')
+    @mock.patch.object(clients, 'create_neutron_client')
     def test_create(self, mock_neutron_client, mock_nova_client,
                     mock_check_quotas, mock_db_create,
                     mock_ins_create, mock_task_api):
@@ -230,12 +253,39 @@ class MongoDBClusterTest(trove_testtools.TestCase):
                       mock_ins_create.call_args_list].count(nics)
         self.assertEqual(7, nics_count)
 
+    @mock.patch.object(task_api, 'load')
+    @mock.patch.object(models.DBCluster, 'create')
+    @mock.patch.object(models, 'validate_instance_nics')
+    @mock.patch.object(QUOTAS, 'check_quotas')
+    @mock.patch.object(models, 'validate_instance_flavors')
+    @mock.patch.object(inst_models.Instance, 'create')
+    def test_create_with_extended_properties(self, mock_ins_create, *args):
+        extended_properties = {
+            "num_configsvr": 5,
+            "num_mongos": 7,
+            "configsvr_volume_size": 8,
+            "configsvr_volume_type": "foo_type",
+            "mongos_volume_size": 9,
+            "mongos_volume_type": "bar_type"}
+        self.cluster.create(mock.Mock(),
+                            self.cluster_name,
+                            self.datastore,
+                            self.datastore_version,
+                            self.instances,
+                            extended_properties, None, None)
+        volume_args_list = [
+            (arg[8], kw['volume_type']) for arg, kw in
+            mock_ins_create.call_args_list
+        ]
+        self.assertEqual(5, volume_args_list.count((8, "foo_type")))
+        self.assertEqual(7, volume_args_list.count((9, "bar_type")))
+
     @mock.patch.object(task_api, 'load')
     @mock.patch.object(inst_models.Instance, 'create')
     @mock.patch.object(models.DBCluster, 'create')
     @mock.patch.object(QUOTAS, 'check_quotas')
-    @mock.patch.object(remote, 'create_nova_client')
-    @mock.patch.object(remote, 'create_neutron_client')
+    @mock.patch.object(clients, 'create_nova_client')
+    @mock.patch.object(clients, 'create_neutron_client')
     @mock.patch.object(api, 'CONF')
     def test_create_with_lower_configsvr(self, mock_conf, mock_neutron_client,
                                          mock_nova_client, ock_check_quotas,
@@ -261,8 +311,8 @@ class MongoDBClusterTest(trove_testtools.TestCase):
     @mock.patch.object(inst_models.Instance, 'create')
     @mock.patch.object(models.DBCluster, 'create')
     @mock.patch.object(QUOTAS, 'check_quotas')
-    @mock.patch.object(remote, 'create_nova_client')
-    @mock.patch.object(remote, 'create_neutron_client')
+    @mock.patch.object(clients, 'create_nova_client')
+    @mock.patch.object(clients, 'create_neutron_client')
     @mock.patch.object(api, 'CONF')
     def test_create_with_higher_configsvr(self, mock_conf, mock_neutron_client,
                                           mock_nova_client, mock_check_quotas,
@@ -288,8 +338,8 @@ class MongoDBClusterTest(trove_testtools.TestCase):
     @mock.patch.object(inst_models.Instance, 'create')
     @mock.patch.object(models.DBCluster, 'create')
     @mock.patch.object(QUOTAS, 'check_quotas')
-    @mock.patch.object(remote, 'create_nova_client')
-    @mock.patch.object(remote, 'create_neutron_client')
+    @mock.patch.object(clients, 'create_nova_client')
+    @mock.patch.object(clients, 'create_neutron_client')
     @mock.patch.object(api, 'CONF')
     def test_create_with_higher_mongos(self, mock_conf, mock_neutron_client,
                                        mock_nova_client, mock_check_quotas,
