@@ -26,15 +26,14 @@ from trove.instance import models
 from trove.instance.models import DBInstance
 from trove.instance.models import DBInstanceFault
 from trove.instance.models import Instance
+from trove.instance.models import instance_encryption_key_cache
 from trove.instance.models import InstanceServiceStatus
 from trove.instance.models import SimpleInstance
-from trove.instance.models import instance_encryption_key_cache
 from trove.instance.service_status import ServiceStatuses
 from trove.instance.tasks import InstanceTasks
 from trove.taskmanager import api as task_api
 from trove.tests.fakes import nova
 from trove.tests.unittests import trove_testtools
-from trove.tests.unittests.util import util
 
 CONF = cfg.CONF
 
@@ -117,7 +116,6 @@ class CreateInstanceTest(trove_testtools.TestCase):
 
     @patch.object(task_api.API, 'get_client', Mock(return_value=Mock()))
     def setUp(self):
-        util.init_db()
         self.context = trove_testtools.TroveTestContext(self, is_admin=True)
         self.name = "name"
         self.flavor_id = 5
@@ -136,6 +134,8 @@ class CreateInstanceTest(trove_testtools.TestCase):
                 manager="mysql",
                 image_id="image_id",
                 packages="",
+                registry_ext="registry_ext",
+                repl_strategy="repl_strategy",
                 active=True))
         self.volume_size = 1
         self.az = "az"
@@ -250,7 +250,6 @@ class TestInstanceUpgrade(trove_testtools.TestCase):
 
     def setUp(self):
         self.context = trove_testtools.TroveTestContext(self, is_admin=True)
-        util.init_db()
 
         self.datastore = datastore_models.DBDatastore.create(
             id=str(uuid.uuid4()),
@@ -264,6 +263,8 @@ class TestInstanceUpgrade(trove_testtools.TestCase):
             packages=str(uuid.uuid4()),
             datastore_id=self.datastore.id,
             manager='test',
+            registry_ext="registry_ext",
+            repl_strategy="repl_strategy",
             active=1)
 
         self.datastore_version2 = datastore_models.DBDatastoreVersion.create(
@@ -273,6 +274,8 @@ class TestInstanceUpgrade(trove_testtools.TestCase):
             packages=str(uuid.uuid4()),
             datastore_id=self.datastore.id,
             manager='test',
+            registry_ext="registry_ext",
+            repl_strategy="repl_strategy",
             active=1)
 
         self.safe_nova_client = clients.create_nova_client
@@ -323,7 +326,6 @@ class TestInstanceUpgrade(trove_testtools.TestCase):
 class TestReplication(trove_testtools.TestCase):
 
     def setUp(self):
-        util.init_db()
 
         self.datastore = datastore_models.DBDatastore.create(
             id=str(uuid.uuid4()),
@@ -337,6 +339,8 @@ class TestReplication(trove_testtools.TestCase):
             packages=str(uuid.uuid4()),
             datastore_id=self.datastore.id,
             manager='mysql',
+            registry_ext="registry_ext",
+            repl_strategy="repl_strategy",
             active=1)
 
         self.databases = []
