@@ -14,10 +14,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from mock import Mock, patch, PropertyMock
-from proboscis.asserts import assert_equal
+from unittest.mock import MagicMock, Mock, patch, PropertyMock
 
 from trove.backup.models import Backup
+from trove.common import cfg
 from trove.common.exception import TroveError, ReplicationSlaveAttachError
 from trove.common import server_group as srv_grp
 from trove.instance.tasks import InstanceTasks
@@ -26,6 +26,8 @@ from trove.taskmanager import models
 from trove.taskmanager import service
 from trove.tests.unittests import trove_testtools
 
+CONF = cfg.CONF
+
 
 class TestManager(trove_testtools.TestCase):
 
@@ -61,10 +63,10 @@ class TestManager(trove_testtools.TestCase):
             with patch.object(self.manager, '_get_replica_txns',
                               return_value=txn_list):
                 result = self.manager._most_current_replica(master, None)
-                assert_equal(result, selected_master)
+                self.assertEqual(result, selected_master)
 
-        with self.assertRaisesRegexp(TroveError,
-                                     'not all replicating from same'):
+        with self.assertRaisesRegex(TroveError,
+                                    'not all replicating from same'):
             test_case([['a', '2a99e-32bf', 2], ['b', '2a', 1]], None)
 
         test_case([['a', '2a99e-32bf', 2]], 'a')
@@ -94,12 +96,13 @@ class TestManager(trove_testtools.TestCase):
         self.mock_slave1.detach_replica.assert_called_with(
             self.mock_old_master, for_failover=True)
         self.mock_old_master.attach_replica.assert_called_with(
-            self.mock_slave1)
+            self.mock_slave1, restart=False)
         self.mock_slave1.make_read_only.assert_called_with(False)
 
         self.mock_slave2.detach_replica.assert_called_with(
             self.mock_old_master, for_failover=True)
-        self.mock_slave2.attach_replica.assert_called_with(self.mock_slave1)
+        self.mock_slave2.attach_replica.assert_called_with(self.mock_slave1,
+                                                           restart=True)
 
         self.mock_old_master.demote_replication_master.assert_any_call()
 
@@ -140,7 +143,7 @@ class TestManager(trove_testtools.TestCase):
     @patch('trove.taskmanager.manager.LOG')
     def test_exception_TroveError_eject_replica_source(
             self, mock_logging, mock_most_current_replica,
-            mock_set_tast_status):
+            mock_set_task_status):
         self.mock_slave2.detach_replica = Mock(side_effect=TroveError)
         mock_most_current_replica.return_value = self.mock_slave1
         with patch.object(models.BuiltInstanceTasks, 'load',
@@ -158,9 +161,9 @@ class TestManager(trove_testtools.TestCase):
         with patch.object(models.BuiltInstanceTasks, 'load',
                           side_effect=[self.mock_slave1, self.mock_old_master,
                                        self.mock_slave2]):
-            self.assertRaisesRegexp(RuntimeError, 'Error',
-                                    self.manager.promote_to_replica_source,
-                                    self.context, 'some-inst-id')
+            self.assertRaisesRegex(RuntimeError, 'Error',
+                                   self.manager.promote_to_replica_source,
+                                   self.context, 'some-inst-id')
 
     @patch('trove.taskmanager.manager.LOG')
     def test_error_demote_replication_master_promote_to_replica_source(
@@ -178,16 +181,16 @@ class TestManager(trove_testtools.TestCase):
     @patch.object(Manager, '_set_task_status')
     @patch.object(Manager, '_most_current_replica')
     def test_error_eject_replica_source(self, mock_most_current_replica,
-                                        mock_set_tast_status):
+                                        mock_set_task_status):
         self.mock_slave2.detach_replica = Mock(
             side_effect=RuntimeError('Error'))
         mock_most_current_replica.return_value = self.mock_slave1
         with patch.object(models.BuiltInstanceTasks, 'load',
                           side_effect=[self.mock_master, self.mock_slave1,
                                        self.mock_slave2]):
-            self.assertRaisesRegexp(RuntimeError, 'Error',
-                                    self.manager.eject_replica_source,
-                                    self.context, 'some-inst-id')
+            self.assertRaisesRegex(RuntimeError, 'Error',
+                                   self.manager.eject_replica_source,
+                                   self.context, 'some-inst-id')
 
     @patch.object(Backup, 'delete')
     @patch.object(models.BuiltInstanceTasks, 'load')
@@ -207,8 +210,34 @@ class TestManager(trove_testtools.TestCase):
                                          'some-master-id', None, None,
                                          None, None)
         mock_tasks.get_replication_master_snapshot.assert_called_with(
-            self.context, 'some-master-id', mock_flavor, 'temp-backup-id',
-            replica_number=1)
+            self.context, 'some-master-id', mock_flavor,
+            parent_backup_id='temp-backup-id', snapshot_driver='swift')
+        mock_backup_delete.assert_called_with(self.context, 'test-id')
+
+    @patch.object(Backup, 'delete')
+    @patch.object(models.BuiltInstanceTasks, 'load')
+    def test_create_replication_cinder(self, mock_load, mock_backup_delete):
+        CONF.set_override('replica_snapshot_driver', 'cinder')
+        mock_tasks = Mock()
+        mock_snapshot = {'dataset': {'snapshot_id': 'test-id'}}
+        mock_tasks.get_replication_master_snapshot = Mock(
+            return_value=mock_snapshot)
+        mock_flavor = Mock()
+        mock_volume = mock_tasks.volume_client.volumes.get.return_value
+        mock_volume.availability_zone = 'nova'
+
+        mock_tasks.get_instance()
+        with patch.object(models.FreshInstanceTasks, 'load',
+                          return_value=mock_tasks):
+            self.manager._create_replication_slave(
+                self.context, ['id_slave'], 'mysql-server',
+                mock_flavor, Mock(), None, None,
+                'mysql', None, 2, 'nova', ['password'], None,
+                None, 'some-master-id', None, None, None, None, None)
+
+        mock_tasks.get_replication_master_snapshot.assert_called_with(
+            self.context, 'some-master-id', mock_flavor,
+            parent_backup_id=None, snapshot_driver='cinder')
         mock_backup_delete.assert_called_with(self.context, 'test-id')
 
     @patch.object(models.FreshInstanceTasks, 'load')
@@ -225,7 +254,7 @@ class TestManager(trove_testtools.TestCase):
                           Mock(), 'some-master-id', None, None, None, None)
 
     def test_AttributeError_create_instance(self):
-        self.assertRaisesRegexp(
+        self.assertRaisesRegex(
             AttributeError, 'Cannot create multiple non-replica instances.',
             self.manager.create_instance, self.context, ['id1', 'id2'],
             Mock(), Mock(), Mock(), None, None, 'mysql', 'mysql-server', 2,
@@ -247,16 +276,19 @@ class TestManager(trove_testtools.TestCase):
                     'mysql-image-id', None, None, 'mysql', 'mysql-server', 2,
                     'temp-backup-id', None, 'password', None, mock_override,
                     None, None, None, None, 'affinity')
-        mock_tasks.create_instance.assert_called_with(mock_flavor,
-                                                      'mysql-image-id', None,
-                                                      None, 'mysql',
-                                                      'mysql-server', 2,
-                                                      'temp-backup-id', None,
-                                                      'password', None,
-                                                      mock_override,
-                                                      None, None, None, None,
-                                                      {'group': 'sg-id'})
-        mock_tasks.wait_for_instance.assert_called_with(36000, mock_flavor)
+
+        mock_tasks.create_instance.assert_called_with(
+            mock_flavor,
+            'mysql-image-id', None,
+            None, 'mysql',
+            'mysql-server', 2,
+            'temp-backup-id', None,
+            'password', None,
+            mock_override,
+            None, None, None, None,
+            {'group': 'sg-id'},
+            access=None, ds_version=None)
+        mock_tasks.wait_for_instance.assert_called_with(3600, mock_flavor)
 
     def test_create_cluster(self):
         mock_tasks = Mock()
@@ -274,6 +306,42 @@ class TestManager(trove_testtools.TestCase):
         mock_tasks.delete_cluster.assert_called_with(self.context,
                                                      'some-cluster-id')
 
+    def test_shrink_cluster_with_success(self):
+        self._assert_shrink_cluster(True)
+
+    def test_shrink_cluster_with_error(self):
+        self._assert_shrink_cluster(False)
+
+    @patch('trove.taskmanager.manager.EndNotification')
+    @patch('trove.taskmanager.manager.models.load_cluster_tasks')
+    def _assert_shrink_cluster(self, success, mock_load, mock_notification):
+        if success:
+            mock_load.side_effect = Mock()
+        else:
+            mock_load.side_effect = Exception
+
+        end_notification = MagicMock()
+        mock_notification.return_value = end_notification
+        context = Mock()
+        cluster_id = Mock()
+        instance_ids = Mock()
+
+        try:
+            self.manager.shrink_cluster(context, cluster_id, instance_ids)
+            self.assertTrue(success)
+        except Exception:
+            self.assertFalse(success)
+
+        mock_load.assert_called_once_with(context, cluster_id)
+        mock_notification.assert_called_once_with(context,
+                                                  cluster_id=cluster_id,
+                                                  instance_ids=instance_ids)
+        exit_error_type = end_notification.__exit__.call_args_list[0][0][0]
+        if success:
+            self.assertFalse(exit_error_type)
+        else:
+            self.assertTrue(exit_error_type)
+
 
 class TestTaskManagerService(trove_testtools.TestCase):
     def test_app_factory(self):
