@@ -51,7 +51,6 @@ from trove.common import timeutils
 from trove.common import utils
 from trove.datastore import models as datastore_models
 from trove.extensions.common import models as common_models
-from trove.extensions.mysql import models as mysql_models
 from trove.instance.models import BaseInstance
 from trove.instance.models import DBInstance
 from trove.instance.models import InstanceServiceStatus
@@ -453,7 +452,7 @@ class FreshInstanceTasksTest(BaseFreshInstanceTasksTest):
 
         mock_build_volume_info.assert_called_with(
             'mysql', availability_zone=None, volume_size=2,
-            volume_type='volume_type'
+            volume_type='volume_type', snapshot_id=None
         )
         mock_guest_prepare.assert_called_with(
             768, mock_build_volume_info(), 'mysql-server', None, None, None,
@@ -537,7 +536,7 @@ class FreshInstanceTasksTest(BaseFreshInstanceTasksTest):
             image_id = None
         mock_build_volume_info.assert_called_with(
             'mysql', availability_zone=None, volume_size=2,
-            volume_type='volume_type'
+            volume_type='volume_type', snapshot_id=None
         )
         mock_guest_prepare.assert_called_with(
             768, mock_build_volume_info(), 'mysql-server', None, None, None,
@@ -818,9 +817,17 @@ class BuiltInstanceTasksTest(trove_testtools.TestCase):
         # this is used during the final check of whether the resize successful
         db_instance.server_status = 'HEALTHY'
         self.db_instance = db_instance
+        datastore_version = MagicMock(
+            spec=datastore_models.DatastoreVersion)
+        datastore_version.id = '1'
+        datastore_version.name = '5.7'
+        datastore_version.version = '5.7'
+        datastore_version.datastore_id = 'id-1'
+        datastore_version.manager = 'mysql'
+        datastore_version.datastore_name = 'mysql'
         self.dm_dv_load_by_uuid_patch = patch.object(
             datastore_models.DatastoreVersion, 'load_by_uuid', MagicMock(
-                return_value=datastore_models.DatastoreVersion(db_instance)))
+                return_value=datastore_version))
         self.dm_dv_load_by_uuid_mock = self.dm_dv_load_by_uuid_patch.start()
         self.addCleanup(self.dm_dv_load_by_uuid_patch.stop)
         self.dm_ds_load_patch = patch.object(
@@ -1094,6 +1101,7 @@ class BackupTasksTest(trove_testtools.TestCase):
         self.backup.updated = 'today'
         self.backup.size = 2.0
         self.backup.state = state.BackupState.NEW
+        self.backup.storage_driver = 'backup'
         self.bm_backup_patches = patch.multiple(
             backup_models.Backup,
             delete=MagicMock(return_value=None),
@@ -1211,7 +1219,7 @@ class RootReportTest(trove_testtools.TestCase):
     def test_report_root_first_time(self):
         context = Mock()
         context.user_id = utils.generate_uuid()
-        report = mysql_models.RootHistory.create(
+        report = common_models.RootHistory.create(
             context, utils.generate_uuid())
         self.assertIsNotNone(report)
 
@@ -1219,11 +1227,11 @@ class RootReportTest(trove_testtools.TestCase):
         context = Mock()
         context.user_id = utils.generate_uuid()
         id = utils.generate_uuid()
-        history = mysql_models.RootHistory(id, context.user_id).save()
-        with patch.object(mysql_models.RootHistory, 'load',
+        history = common_models.RootHistory(id, context.user_id).save()
+        with patch.object(common_models.RootHistory, 'load',
                           Mock(return_value=history)):
-            report = mysql_models.RootHistory.create(context, id)
-            self.assertTrue(mysql_models.RootHistory.load.called)
+            report = common_models.RootHistory.create(context, id)
+            self.assertTrue(common_models.RootHistory.load.called)
             self.assertEqual(history.user, report.user)
             self.assertEqual(history.id, report.id)
 
